# -*- coding: utf-8 -*-

##############################################################################
# TTYLight-Python: easily stylize terminal text
##############################################################################
#
# Author: Travis Cardwell <travis.cardwell@yuzutechnology.com>
# Copyright: Copyright (c) 2013, Yuzu Technology, Inc.
#
# This file is subject to the terms and conditions defined in the License
# file included with this source code.
#
##############################################################################

"""\
ttylight Library
================

The :mod:`ttylight` library encodes
`ANSI escape codes <http://en.wikipedia.org/wiki/ANSI_escape_code>`_ to set
text color and intensity.  Common usage is as follows::

    import ttylight

    print("This is an example of " +
          ttylight.stylize("highlighted", fg=ttylight.Color.RED) +
          " text.")

This example uses the :func:`stylize` function to print the string
`"This is an example of highlighted text."` to the terminal with the word
`"highlighted"` in red.  The foreground and background colors can be specified
using :class:`Color` constants, and the text intensity can be specified using
:class:`Intensity` constants.

In addition, the :func:`set_terminal_title` function is used to set the
terminal title.

API
---
"""

import sys


__version__ = '1.0.1'


class Color:
    """Color constants

    This class is used as a namespace and is not meant to be instantiated.
    Access the color constants as class variables::

        import ttylight

        print(ttylight.stylize("hello", fg=ttylight.Color.RED))

    :var int BLACK: black
    :var int RED: red
    :var int GREEN: green
    :var int YELLOW: yellow
    :var int BLUE: blue
    :var int MAGENTA: magenta
    :var int CYAN: cyan
    :var int WHITE: white
    :var int DEFAULT: default
    """

    BLACK = 0
    RED = 1
    GREEN = 2
    YELLOW = 3
    BLUE = 4
    MAGENTA = 5
    CYAN = 6
    WHITE = 7
    DEFAULT = 9


class Intensity:
    """Intensity constants

    This class is used as a namespace and is not meant to be instantiated.
    Access the intensity constants as class variables::

        import ttylight

        print(ttylight.stylize("hello", intensity=ttylight.Intensity.BOLD))

    :var int NORMAL: normal
    :var int BOLD: bold
    :var int FAINT: faint
    """

    NORMAL = 0
    BOLD = 1
    FAINT = 2


def encode(fg=None, bg=None, intensity=None):
    """Encode an ANSI escape sequence

    This is a relatively low-level function that encodes a given foreground
    :class:`Color`, background :class:`Color`, and :class:`Intensity`.  Use it
    if you need to change between styles without resetting.

    If you would like to stylize some text and then reset, consider using the
    :func:`stylize` function instead.

    :param int fg: foreground color (Color constant)
    :param int bg: background color (Color constant)
    :param int intensity: intensity (Intensity constant)
    :returns: encoded ANSI escape sequence
    :rtype: str
    """
    codes = []
    if intensity is not None:
        codes.append(str(intensity))
    if fg is not None:
        codes.append(str(30 + fg))
    if bg is not None:
        codes.append(str(40 + bg))
    if len(codes) == 0:
        return ''
    return '\033[' + ';'.join(codes) + 'm'


def reset():
    """Return the ANSI reset sequence

    This is a relatively low-level function.  Use it to reset the style after
    using the :func:`encode` function.

    If you would like to stylize some text and then reset, consider using the
    :func:`stylize` function instead.

    :returns: the ANSI reset sequence
    :rtype: str
    """
    return '\033[0m'


def stylize(text, fg=None, bg=None, intensity=None):
    """Stylize text

    This is a high-level function that stylizes text with the given foreground
    :class:`Color`, background :class:`Color`, and :class:`Intensity` and then
    resets the style::

        import ttylight

        print(ttylight.stylize("hello", fg=ttylight.Color.YELLOW,
                                        bg=ttylight.Color.RED,
                                        intensity=ttylight.Intensity.BOLD))

    :param str text: text to stylize
    :param int fg: foreground color (Color constant)
    :param int bg: background color (Color constant)
    :param int intensity: intensity (Intensity constant)
    :returns: text with ANSI escape sequences
    :rtype: str
    """
    style = encode(fg, bg, intensity)
    if text == '' or style == '':
        return text
    return style + text + reset()


def set_terminal_title(msg):
    """Set the terminal title

    The message should not contain ANSI sequences.

    :param str msg: message to write
    """
    sys.stdout.write('\033]0;{:s}\007'.format(msg.strip()))
    sys.stdout.flush()
